funcprot(0);
stacksize('max');

//座標値は全て複素平面において複素数を用いて示される。

function w = F(z,x,q)
//F 代用電荷法の近似関数
//z 引数、x 電荷点座標、q 電荷
// x = [x1,x2,...], q=[q0,q1,...] のように1×n行列で格納される。
//w 近似関数値
q0 = q(1); //定数関数の係数
qq = q(2:size(q,'*')); //定数関数以外の基底関数の係数
w = q0*ones(z); //返値が引数の行列と同サイズになるように代入
for i = 1:size(x,'*')
  w = w + qq(i)*log(abs(z-x(i)*ones(z)));
end
endfunction

function [x,y] = SetupSquare(z1,z2,xn,yn)
// SetupSquare 矩形領域の問題に必要な設定
// z1 z2 領域の対角頂点座標、xn yn x方向・y方向の辺の分割数

// 拘束点(補間点)座標yのために、左下隅を起点に左回りに座標列を取る
xmin = min(real(z1),real(z2));
xmax = max(real(z1),real(z2));
ymin = min(imag(z1),imag(z2));
ymax = max(imag(z1),imag(z2));
xwid = xmax - xmin;
ywid = ymax - ymin;
y = [xmin+[0:xn-1]*xwid/xn+%i*ymin];
y = [y,xmax+%i*(ymin+[0:yn-1]*ywid/yn)];
y = [y,xmax-[0:xn-1]*xwid/xn+%i*ymax];
y = [y,xmin+%i*(ymax-[0:yn-1]*ywid/yn)];

// 境界拡大法と呼ばれる簡易な方法で電荷点を生成する。
// 電荷点(特異点)座標のために、拘束点(補間点)を中心点から1.5倍に拡大する
y0 = (xmax+xmin)/2 + %i*(ymax+ymin)/2;
x = (y - y0)*1.5 + y0;
endfunction

function A = Amat(x,y)
//Amat 代用電荷法の係数行列を計算する
//x 電荷点(特異点)座標、y 拘束点(補間点)座標
//A 係数行列
A = [0,ones(x); ones(y)',log(abs(ones(y)'*x-y.'*ones(x)))];
endfunction

function w = Bfunc(z)
//Bfunc 境界値関数
//z 引数(複素平面座標)
//w 境界値
w = zeros(z);
// zの実部が0の成分だけを再計算
w(real(z)==0) = imag(z(real(z)==0)).*(1-imag(z(real(z)==0)));
endfunction

//メインルーチン
[x,y] = SetupSquare(0,1+%i,16,16);
b = [0; Bfunc(y).'];
A = Amat(x,y);
q = A\b;
//縦横n等分した点での値を用いてグラフ表示する（grayplotの座標のとりかたに注意）
n=1000; z = ones(n+1,1)*linspace(0,1,n+1)+linspace(0,1,n+1).'*ones(1,n+1)*%i;
w = F(z,x,q);
grayplot(linspace(0,1,n+1),linspace(0,1,n+1),w');